@@ -31,12 +31,14 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <xkbcommon/xkbcommon-compose.h>
 
 #include "fcitx/fcitx.h"
 #include "fcitx-utils/log.h"
 #include "fcitx/frontend.h"
 
 #include "fcitximcontext.h"
+#include "fcitx-gclient/fcitxconnection.h"
 #include "fcitx-gclient/fcitxclient.h"
 
 #if !GTK_CHECK_VERSION (2, 91, 0)
@@ -47,6 +49,25 @@
 # define NEW_GDK_WINDOW_GET_DISPLAY
 #endif
 
+static const FcitxCapacityFlags purpose_related_capacity =
+    CAPACITY_ALPHA |
+    CAPACITY_DIGIT |
+    CAPACITY_NUMBER |
+    CAPACITY_DIALABLE |
+    CAPACITY_URL |
+    CAPACITY_EMAIL |
+    CAPACITY_PASSWORD;
+
+static const FcitxCapacityFlags hints_related_capacity =
+    CAPACITY_SPELLCHECK |
+    CAPACITY_NO_SPELLCHECK |
+    CAPACITY_WORD_COMPLETION |
+    CAPACITY_LOWERCASE |
+    CAPACITY_UPPERCASE |
+    CAPACITY_UPPERCASE_WORDS |
+    CAPACITY_UPPERCASE_SENTENCES |
+    CAPACITY_NO_ON_SCREEN_KEYBOARD;
+
 struct _FcitxIMContext {
     GtkIMContext parent;
 
@@ -57,14 +78,17 @@ struct _FcitxIMContext {
     int has_focus;
     guint32 time;
     gboolean use_preedit;
+    gboolean support_surrounding_text;
     gboolean is_inpreedit;
     gchar* preedit_string;
     gchar* surrounding_text;
     int cursor_pos;
-    FcitxCapacityFlags capacity;
+    FcitxCapacityFlags capacity_from_toolkit;
+    FcitxCapacityFlags last_updated_capacity;
     PangoAttrList* attrlist;
     gint last_cursor_pos;
     gint last_anchor_pos;
+    struct xkb_compose_state* xkbComposeState;
 };
 
 struct _FcitxIMContextClass {
@@ -182,6 +206,9 @@ static GtkIMContext *_focus_im_context = NULL;
 static const gchar *_no_snooper_apps = NO_SNOOPER_APPS;
 static gboolean _use_key_snooper = _ENABLE_SNOOPER;
 static guint    _key_snooper_id = 0;
+static FcitxConnection* _connection = NULL;
+static struct xkb_context* xkbContext = NULL;
+static struct xkb_compose_table* xkbComposeTable = NULL;
 
 /* Copied from gtk+2.0-2.20.1/modules/input/imcedilla.c to fix crosbug.com/11421.
 * Overwrite the original Gtk+'s compose table in gtk+-2.x.y/gtk/gtkimcontextsimple.c. */
@@ -364,7 +391,7 @@ fcitx_im_context_init(FcitxIMContext *context)
     context->last_cursor_pos = -1;
     context->preedit_string = NULL;
     context->attrlist = NULL;
-    context->capacity = CAPACITY_SURROUNDING_TEXT;
+    context->last_updated_capacity = CAPACITY_SURROUNDING_TEXT;
 
     context->slave = gtk_im_context_simple_new();
     gtk_im_context_simple_add_table(GTK_IM_CONTEXT_SIMPLE(context->slave),
@@ -404,7 +431,31 @@ fcitx_im_context_init(FcitxIMContext *context)
 
     context->time = GDK_CURRENT_TIME;
 
-    context->client = fcitx_client_new();
+    static gsize has_info = 0;
+    if (g_once_init_enter(&has_info)) {
+        _connection = fcitx_connection_new();
+        g_object_ref_sink(_connection);
+
+        xkbContext = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
+
+        if (xkbContext) {
+            xkb_context_set_log_level(xkbContext, XKB_LOG_LEVEL_CRITICAL);
+        }
+
+        const char* locale = getenv("LC_ALL");
+        if (!locale)
+            locale = getenv("LC_CTYPE");
+        if (!locale)
+            locale = getenv("LANG");
+        if (!locale)
+            locale = "C";
+
+        xkbComposeTable = xkbContext ? xkb_compose_table_new_from_locale(xkbContext, locale, XKB_COMPOSE_COMPILE_NO_FLAGS) : NULL;
+
+        g_once_init_leave(&has_info, 1);
+    }
+
+    context->client = fcitx_client_new_with_connection(_connection);
     g_signal_connect(context->client, "connected", G_CALLBACK(_fcitx_im_context_connect_cb), context);
     g_signal_connect(context->client, "enable-im", G_CALLBACK(_fcitx_im_context_enable_im_cb), context);
     g_signal_connect(context->client, "close-im", G_CALLBACK(_fcitx_im_context_close_im_cb), context);
@@ -412,6 +463,8 @@ fcitx_im_context_init(FcitxIMContext *context)
     g_signal_connect(context->client, "commit-string", G_CALLBACK(_fcitx_im_context_commit_string_cb), context);
     g_signal_connect(context->client, "delete-surrounding-text", G_CALLBACK(_fcitx_im_context_delete_surrounding_text_cb), context);
     g_signal_connect(context->client, "update-formatted-preedit", G_CALLBACK(_fcitx_im_context_update_formatted_preedit_cb), context);
+
+    context->xkbComposeState = xkbComposeTable ? xkb_compose_state_new(xkbComposeTable, XKB_COMPOSE_STATE_NO_FLAGS) : NULL;
 }
 
 static void
@@ -427,6 +480,11 @@ fcitx_im_context_finalize(GObject *obj)
     g_signal_handlers_disconnect_matched ((instance), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, (data))
 #endif
 
+    if (context->xkbComposeState) {
+        xkb_compose_state_unref(context->xkbComposeState);
+        context->xkbComposeState = NULL;
+    }
+
     if (context->client) {
         g_signal_handlers_disconnect_by_data(context->client, context);
         g_object_unref(context->client);
@@ -484,6 +542,38 @@ fcitx_im_context_set_client_window(GtkIMContext          *context,
     set_ic_client_window(fcitxcontext, client_window);
 }
 
+static gboolean
+fcitx_im_context_filter_keypress_fallback(FcitxIMContext *context, GdkEventKey *event)
+{
+    if (!context->xkbComposeState || event->type == GDK_KEY_RELEASE) {
+        return gtk_im_context_filter_keypress(context->slave, event);;
+    }
+
+    struct xkb_compose_state* xkbComposeState = context->xkbComposeState;
+
+    enum xkb_compose_feed_result result = xkb_compose_state_feed(xkbComposeState, event->keyval);
+    if (result == XKB_COMPOSE_FEED_IGNORED) {
+        return gtk_im_context_filter_keypress(context->slave, event);
+    }
+
+    enum xkb_compose_status status = xkb_compose_state_get_status(xkbComposeState);
+    if (status == XKB_COMPOSE_NOTHING) {
+        return gtk_im_context_filter_keypress(context->slave, event);
+    } else if (status == XKB_COMPOSE_COMPOSED) {
+        char buffer[] = {'\0', '\0', '\0', '\0', '\0', '\0', '\0'};
+        int length = xkb_compose_state_get_utf8(xkbComposeState, buffer, sizeof(buffer));
+        xkb_compose_state_reset(xkbComposeState);
+        if (length != 0) {
+            g_signal_emit(context, _signal_commit_id, 0, buffer);
+        }
+
+    } else if (status == XKB_COMPOSE_CANCELLED) {
+        xkb_compose_state_reset(xkbComposeState);
+    }
+
+    return TRUE;
+}
+
 ///
 static gboolean
 fcitx_im_context_filter_keypress(GtkIMContext *context,
@@ -511,7 +601,7 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
         return TRUE;
 
     if (G_UNLIKELY(event->state & FcitxKeyState_IgnoredMask))
-        return gtk_im_context_filter_keypress(fcitxcontext->slave, event);
+        return fcitx_im_context_filter_keypress_fallback(fcitxcontext, event);
 
     if (fcitx_client_is_valid(fcitxcontext->client) && fcitxcontext->has_focus) {
         _request_surrounding_text (&fcitxcontext);
@@ -529,7 +619,7 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
                                                     event->time);
             if (ret <= 0) {
                 event->state |= FcitxKeyState_IgnoredMask;
-                return gtk_im_context_filter_keypress(fcitxcontext->slave, event);
+                return fcitx_im_context_filter_keypress_fallback(fcitxcontext, event);
             } else {
                 event->state |= FcitxKeyState_HandledMask;
                 return TRUE;
@@ -550,7 +640,7 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
             return TRUE;
         }
     } else {
-        return gtk_im_context_filter_keypress(fcitxcontext->slave, event);
+        return fcitx_im_context_filter_keypress_fallback(fcitxcontext, event);
     }
     return FALSE;
 }
@@ -622,9 +712,31 @@ _fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
                                           (gpointer *)&widget);
                 if (GTK_IS_WIDGET(widget)) {
                     hasColor = true;
+#if GTK_CHECK_VERSION (3, 0, 0)
+                    GtkStyleContext* styleContext = gtk_widget_get_style_context(widget);
+                    GdkRGBA* fg_rgba = NULL;
+                    GdkRGBA* bg_rgba = NULL;
+                    gtk_style_context_get(styleContext, GTK_STATE_FLAG_SELECTED,
+                                          "background-color", &bg_rgba,
+                                          "color", &fg_rgba,
+                                          NULL
+                                         );
+
+                    fg.pixel = 0;
+                    fg.red = CLAMP ((guint) (fg_rgba->red * 65535), 0, 65535);
+                    fg.green = CLAMP ((guint) (fg_rgba->green * 65535), 0, 65535);
+                    fg.blue = CLAMP ((guint) (fg_rgba->blue * 65535), 0, 65535);
+                    bg.pixel = 0;
+                    bg.red = CLAMP ((guint) (bg_rgba->red * 65535), 0, 65535);
+                    bg.green = CLAMP ((guint) (bg_rgba->green * 65535), 0, 65535);
+                    bg.blue = CLAMP ((guint) (bg_rgba->blue * 65535), 0, 65535);
+                    gdk_rgba_free (fg_rgba);
+                    gdk_rgba_free (bg_rgba);
+#else
                     GtkStyle* style = gtk_widget_get_style(widget);
                     fg = style->text[GTK_STATE_SELECTED];
                     bg = style->bg[GTK_STATE_SELECTED];
+#endif
                 }
             }
 
@@ -700,11 +812,17 @@ fcitx_im_context_focus_in(GtkIMContext *context)
 
     fcitxcontext->has_focus = true;
 
+    /*
+     * Do not call gtk_im_context_focus_out() here.
+     * This might workaround some chrome issue
+     */
+#if 0
     if (_focus_im_context != NULL) {
         g_assert (_focus_im_context != context);
         gtk_im_context_focus_out (_focus_im_context);
         g_assert (_focus_im_context == NULL);
     }
+#endif
 
     if (fcitx_client_is_valid(fcitxcontext->client)) {
         fcitx_client_focus_in(fcitxcontext->client);
@@ -740,7 +858,9 @@ fcitx_im_context_focus_out(GtkIMContext *context)
         return;
     }
 
+#if 0
     g_assert (context == _focus_im_context);
+#endif
     g_object_remove_weak_pointer ((GObject *) context,
                                   (gpointer *) &_focus_im_context);
     _focus_im_context = NULL;
@@ -799,6 +919,7 @@ _set_cursor_location_internal(FcitxIMContext *fcitxcontext)
     }
 
     area = fcitxcontext->area;
+
     if (area.x == -1 && area.y == -1 && area.width == 0 && area.height == 0) {
 #if GTK_CHECK_VERSION (2, 91, 0)
         area.x = 0;
@@ -819,10 +940,18 @@ _set_cursor_location_internal(FcitxIMContext *fcitxcontext)
     {
         int rootx, rooty;
         gdk_window_get_origin(fcitxcontext->client_window, &rootx, &rooty);
-        area.x = rootx + area.x;
-        area.y = rooty + area.y;
+        area.x += rootx;
+        area.y += rooty;
     }
 #endif
+    int scale = 1;
+#if GTK_CHECK_VERSION (3, 10, 0)
+    scale = gdk_window_get_scale_factor(fcitxcontext->client_window);
+#endif
+    area.x *= scale;
+    area.y *= scale;
+    area.width *= scale;
+    area.height *= scale;
 
     fcitx_client_set_cusor_rect(fcitxcontext->client, area.x, area.y, area.width, area.height);
     return FALSE;
@@ -924,7 +1053,7 @@ fcitx_im_context_set_surrounding (GtkIMContext *context,
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT (context);
 
     if (fcitx_client_is_valid(fcitxcontext->client) &&
-        !(fcitxcontext->capacity & CAPACITY_PASSWORD)) {
+        !(fcitxcontext->last_updated_capacity & CAPACITY_PASSWORD)) {
         gint cursor_pos;
         guint utf8_len;
         gchar *p;
@@ -960,10 +1089,18 @@ void
 _fcitx_im_context_set_capacity(FcitxIMContext* fcitxcontext, gboolean force)
 {
     if (fcitx_client_is_valid(fcitxcontext->client)) {
-        FcitxCapacityFlags flags = fcitxcontext->capacity & ~(CAPACITY_PREEDIT | CAPACITY_FORMATTED_PREEDIT | CAPACITY_PASSWORD);
-        if (fcitxcontext->use_preedit)
+        FcitxCapacityFlags flags = fcitxcontext->capacity_from_toolkit;
+        // toolkit hint always not have preedit / surrounding hint
+        // no need to check them
+        if (fcitxcontext->use_preedit) {
             flags |= CAPACITY_PREEDIT | CAPACITY_FORMATTED_PREEDIT;
+        }
+        if (fcitxcontext->support_surrounding_text) {
+            flags |= CAPACITY_SURROUNDING_TEXT;
+        }
 
+        // always run this code against all gtk version
+        // seems visibility != PASSWORD hint
         if (fcitxcontext->client_window != NULL) {
             GtkWidget *widget;
             gdk_window_get_user_data (fcitxcontext->client_window,
@@ -975,12 +1112,12 @@ _fcitx_im_context_set_capacity(FcitxIMContext* fcitxcontext, gboolean force)
         }
 
         gboolean update = FALSE;
-        if (G_UNLIKELY(fcitxcontext->capacity != flags)) {
-            fcitxcontext->capacity = flags;
+        if (G_UNLIKELY(fcitxcontext->last_updated_capacity != flags)) {
+            fcitxcontext->last_updated_capacity = flags;
             update = TRUE;
         }
         if (G_UNLIKELY(update || force))
-            fcitx_client_set_capacity(fcitxcontext->client, fcitxcontext->capacity);
+            fcitx_client_set_capacity(fcitxcontext->client, fcitxcontext->last_updated_capacity);
     }
 }
 
@@ -995,6 +1132,10 @@ fcitx_im_context_reset(GtkIMContext *context)
         fcitx_client_reset(fcitxcontext->client);
     }
 
+    if (fcitxcontext->xkbComposeState) {
+        xkb_compose_state_reset(fcitxcontext->xkbComposeState);
+    }
+
     gtk_im_context_reset(fcitxcontext->slave);
 }
 
@@ -1387,12 +1528,12 @@ _request_surrounding_text (FcitxIMContext **context)
         else
             return;
         if (return_value) {
-            (*context)->capacity |= CAPACITY_SURROUNDING_TEXT;
+            (*context)->support_surrounding_text = TRUE;
             _fcitx_im_context_set_capacity (*context,
                                             FALSE);
         }
         else {
-            (*context)->capacity &= ~CAPACITY_SURROUNDING_TEXT;
+            (*context)->support_surrounding_text = FALSE;
             _fcitx_im_context_set_capacity (*context,
                                             FALSE);
         }
@@ -1483,20 +1624,12 @@ _fcitx_im_context_input_purpose_changed_cb(GObject* gobject, GParamSpec* pspec,
     GtkInputPurpose purpose;
     g_object_get(gobject, "input-purpose", &purpose, NULL);
 
-    const FcitxCapacityFlags related_capacity =
-        CAPACITY_ALPHA |
-        CAPACITY_DIGIT |
-        CAPACITY_NUMBER |
-        CAPACITY_DIALABLE |
-        CAPACITY_URL |
-        CAPACITY_EMAIL |
-        CAPACITY_PASSWORD;
 
-    fcitxcontext->capacity &= ~related_capacity;
+    fcitxcontext->capacity_from_toolkit &= ~purpose_related_capacity;
 
 #define CASE_PURPOSE(_PURPOSE, _CAPACITY) \
     case _PURPOSE: \
-        fcitxcontext->capacity |= _CAPACITY; \
+        fcitxcontext->capacity_from_toolkit |= _CAPACITY; \
         break;
 
     switch(purpose) {
@@ -1528,21 +1661,11 @@ _fcitx_im_context_input_hints_changed_cb(GObject* gobject, GParamSpec *pspec,
     GtkInputHints hints;
     g_object_get(gobject, "input-hints", &hints, NULL);
 
-    const FcitxCapacityFlags related_capacity =
-        CAPACITY_SPELLCHECK |
-        CAPACITY_NO_SPELLCHECK |
-        CAPACITY_WORD_COMPLETION |
-        CAPACITY_LOWERCASE |
-        CAPACITY_UPPERCASE |
-        CAPACITY_UPPERCASE_WORDS |
-        CAPACITY_UPPERCASE_SENTENCES |
-        CAPACITY_NO_ON_SCREEN_KEYBOARD;
-
-    fcitxcontext->capacity &= ~related_capacity;
+    fcitxcontext->capacity_from_toolkit &= ~hints_related_capacity;
 
 #define CHECK_HINTS(_HINTS, _CAPACITY) \
     if (hints & _HINTS) \
-        fcitxcontext->capacity |= _CAPACITY;
+        fcitxcontext->capacity_from_toolkit |= _CAPACITY;
 
     CHECK_HINTS(GTK_INPUT_HINT_SPELLCHECK, CAPACITY_SPELLCHECK)
     CHECK_HINTS(GTK_INPUT_HINT_NO_SPELLCHECK, CAPACITY_NO_SPELLCHECK);
