@@ -42,8 +42,10 @@
 struct FcitxQtICData {
     FcitxQtICData() : capacity(0), proxy(0), surroundingAnchor(-1), surroundingCursor(-1) {}
     ~FcitxQtICData() {
-        if (proxy && proxy->isValid()) {
-            proxy->DestroyIC();
+        if (proxy) {
+            if (proxy->isValid()) {
+                proxy->DestroyIC();
+            }
             delete proxy;
         }
     }
