@@ -63,6 +63,9 @@ CONFIG_BINDING_REGISTER("CodeTable", "LangCode", langCode)
 CONFIG_BINDING_REGISTER("CodeTable", "KeyboardLayout", kbdlayout)
 CONFIG_BINDING_REGISTER("CodeTable", "UseCustomPrompt", customPrompt)
 CONFIG_BINDING_REGISTER("CodeTable", "UseAlternativePageKey", bUseAlternativePageKey)
+CONFIG_BINDING_REGISTER("CodeTable", "UseAlternativeCandidateWordNumber", bUseAlternativeCandidateWordNumber)
+CONFIG_BINDING_REGISTER("CodeTable", "CandidateWordNumber", iCandidateWordNumber)
+CONFIG_BINDING_REGISTER("CodeTable", "UseAlternativePageKey", bUseAlternativePageKey)
 CONFIG_BINDING_REGISTER("CodeTable", "AlternativePrevPage", hkAlternativePrevPage)
 CONFIG_BINDING_REGISTER("CodeTable", "AlternativeNextPage", hkAlternativeNextPage)
 CONFIG_BINDING_REGISTER("CodeTable", "FirstCandidateAsPreedit", bFirstCandidateAsPreedit)
@@ -71,6 +74,7 @@ CONFIG_BINDING_REGISTER("CodeTable", "CommitKey", hkCommitKey)
 CONFIG_BINDING_REGISTER("CodeTable", "CommitKeyCommitWhenNoMatch", bCommitKeyCommitWhenNoMatch)
 CONFIG_BINDING_REGISTER("CodeTable", "IgnorePunc", bIgnorePunc)
 CONFIG_BINDING_REGISTER("CodeTable", "IgnorePuncList", ignorePuncList)
+CONFIG_BINDING_REGISTER("CodeTable", "CandidateLayout", layoutHint)
 CONFIG_BINDING_REGISTER("CodeTable", "Enabled", bEnabled)
 CONFIG_BINDING_END()
 
