@@ -141,6 +141,26 @@ extern "C" {
      * @return void
      **/
     void FcitxInstanceRegisterPostInputFilter(struct _FcitxInstance* instance, FcitxKeyFilterHook hook);
+    /**
+     * register pre release input filter
+     *
+     * @param instance fcitx instance
+     * @param hook new hook
+     * @return void
+     *
+     * @since 4.2.8
+     **/
+    void FcitxInstanceRegisterPreReleaseInputFilter(struct _FcitxInstance* instance, FcitxKeyFilterHook hook) ;
+    /**
+     * register post release input filter
+     *
+     * @param instance fcitx instance
+     * @param hook new hook
+     * @return void
+     *
+     * @since 4.2.8
+     **/
+    void FcitxInstanceRegisterPostReleaseInputFilter(struct _FcitxInstance* instance, FcitxKeyFilterHook hook);
     /**
      * register ouput string filter
      *
@@ -264,6 +284,11 @@ extern "C" {
      **/
     void FcitxInstanceRegisterICStateChangedHook(struct _FcitxInstance* instance, FcitxICEventHook hook);
 
+    boolean FcitxDummyReleaseInputHook(void* arg, FcitxKeySym sym,
+                                        unsigned int state,
+                                        INPUT_RETURN_VALUE *retval);
+
+
 #ifdef __cplusplus
 }
 #endif
