@@ -79,6 +79,10 @@ void* UnicodeCreate(FcitxInstance* instance)
     kfhk.func = UnicodePreFilter;
     FcitxInstanceRegisterPreInputFilter(instance, kfhk);
 
+    kfhk.arg = &uni->enable;
+    kfhk.func = FcitxDummyReleaseInputHook;
+    FcitxInstanceRegisterPreReleaseInputFilter(instance, kfhk);
+
     FcitxHotkeyHook hkhk;
     hkhk.arg = uni;
     hkhk.hotkey = uni->key;
@@ -102,7 +106,7 @@ boolean UnicodePreFilter(void* arg, FcitxKeySym sym, unsigned int state,
         FcitxCandidateWordList *candList;
         candList = FcitxInputStateGetCandidateList(input);
 
-        FcitxCandidateWordSetPageSize(candList, 4);
+        FcitxCandidateWordSetPageSize(candList, fc->iMaxCandWord);
         FcitxCandidateWordSetChooseAndModifier(candList, DIGIT_STR_CHOOSE,
                                                FcitxKeyState_Alt);
         if (FcitxHotkeyIsHotKey(sym, state,
@@ -174,7 +178,7 @@ INPUT_RETURN_VALUE UnicodeGetCandWords(UnicodeModule* uni)
     FcitxCandidateWordSetLayoutHint(candList, CLH_Vertical);
 
     UT_array* result = CharSelectDataFind(uni->charselectdata, uni->buffer);
-    utarray_foreach(c, result, uint16_t) {
+    utarray_foreach(c, result, uint32_t) {
         char* s = fcitx_utils_malloc0(sizeof(char) * (UTF8_MAX_LENGTH + 1));
         fcitx_ucs4_to_utf8(*c, s);
         FcitxCandidateWord candWord;
