@@ -160,6 +160,10 @@ void *VKCreate(FcitxInstance* instance)
     hk.func = VKPreFilter;
     FcitxInstanceRegisterPreInputFilter(instance, hk);
 
+    hk.arg = &vkstate->bVK;
+    hk.func = FcitxDummyReleaseInputHook;
+    FcitxInstanceRegisterPreReleaseInputFilter(instance, hk);
+
     FcitxIMEventHook resethk;
     resethk.arg = vkstate;
     resethk.func = VKReset;
@@ -347,7 +351,9 @@ cairo_surface_t* LoadVKImage(VKWindow* vkWindow)
 
     if (!vkWindow->keyboard) {
         char* path = fcitx_utils_get_fcitx_path_with_filename("pkgdatadir", "skin/default/keyboard.png");
-        vkWindow->keyboard = cairo_image_surface_create_from_png(path);
+        if (fcitx_utils_isreg(path)) {
+            vkWindow->keyboard = cairo_image_surface_create_from_png(path);
+        }
         free(path);
     }
     return vkWindow->keyboard;
@@ -383,39 +389,46 @@ void DrawVKWindow(VKWindow* vkWindow)
 
     cr = cairo_create(vkWindow->surface);
     cairo_surface_t* vkimage = LoadVKImage(vkWindow);
-    cairo_set_source_surface(cr, vkimage, 0, 0);
+    if (vkimage) {
+        cairo_set_source_surface(cr, vkimage, 0, 0);
+    } else {
+        cairo_set_source_rgb(cr, 1, 1, 1);
+    }
     cairo_paint(cr);
+
+    FcitxCairoTextContext* ctc = FcitxCairoTextContextCreate(cr);
+    FcitxCairoTextContextSet(ctc, *font, vkWindow->fontSize, 0);
     /* 显示字符 */
     /* 名称 */
-    OutputString(cr, vks[vkstate->iCurrentVK].strName, *font, vkWindow->fontSize, false , (VK_WINDOW_WIDTH - StringWidth(vks[vkstate->iCurrentVK].strName, *font, vkWindow->fontSize, false)) / 2, 6, fontColor);
+    FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strName, (VK_WINDOW_WIDTH - FcitxCairoTextContextStringWidth(ctc, vks[vkstate->iCurrentVK].strName)) / 2, 6, fontColor);
 
     /* 第一排 */
     iPos = 13;
     for (i = 0; i < 13; i++) {
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][1], *font, vkWindow->fontSize, false, iPos, 27, fontColor);
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][0], *font, vkWindow->fontSize, false, iPos - 5, 40, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][1], iPos, 27, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][0], iPos - 5, 40, fontColor);
         iPos += 24;
     }
     /* 第二排 */
     iPos = 48;
     for (i = 13; i < 26; i++) {
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][1], *font, vkWindow->fontSize, false, iPos, 55, fontColor);
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][0], *font, vkWindow->fontSize, false, iPos - 5, 68, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][1], iPos, 55, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][0], iPos - 5, 68, fontColor);
         iPos += 24;
     }
     /* 第三排 */
     iPos = 55;
     for (i = 26; i < 37; i++) {
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][1], *font, vkWindow->fontSize, false, iPos, 83, fontColor);
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][0], *font, vkWindow->fontSize, false, iPos - 5, 96, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][1], iPos, 83, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][0], iPos - 5, 96, fontColor);
         iPos += 24;
     }
 
     /* 第四排 */
     iPos = 72;
     for (i = 37; i < 47; i++) {
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][1], *font, vkWindow->fontSize, false, iPos, 111, fontColor);
-        OutputString(cr, vks[vkstate->iCurrentVK].strSymbol[i][0], *font, vkWindow->fontSize, false, iPos - 5, 124, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][1], iPos, 111, fontColor);
+        FcitxCairoTextContextOutputString(ctc, vks[vkstate->iCurrentVK].strSymbol[i][0], iPos - 5, 124, fontColor);
         iPos += 24;
     }
 
@@ -711,34 +724,28 @@ void SwitchVK(FcitxVKState *vkstate)
 
     if (vkstate->bVK) {
         int             x, y;
-        int dwidth, dheight;
-        FcitxX11GetScreenSize(vkstate->owner, &dwidth, &dheight);
+        FcitxRect rect;
+        int icx = 0, icy = 0, w = 0, h = 0;
+        FcitxInputContext* ic = FcitxInstanceGetCurrentIC(instance);
+        FcitxInstanceGetWindowRect(instance, ic, &icx, &icy, &w, &h);
+        FcitxX11GetScreenGeometry(instance, &icx, &icy, &rect);
+        x = (rect.x1 + rect.x2) / 2 - VK_WINDOW_WIDTH / 2;
+        y = rect.y1 + 40;
 
-        if (!FcitxUISupportMainWindow(instance)) {
-            x = dwidth / 2 - VK_WINDOW_WIDTH / 2;
-            y = 40;
-        } else {
-            int mx = 0, my = 0, mw = 0, mh = 0;
-            FcitxUIGetMainWindowSize(instance, &mx, &my, &mw, &mh);
-            x = mx;
-            y = my + mh + 2;
-            if ((y + VK_WINDOW_HEIGHT) >= dheight)
-                y = my - VK_WINDOW_HEIGHT - 2;
-            if (y < 0)
-                y = 0;
-        }
-        if ((x + VK_WINDOW_WIDTH) >= dwidth)
-            x = dwidth - VK_WINDOW_WIDTH - 1;
-        if (x < 0)
-            x = 0;
+        if ((y + VK_WINDOW_HEIGHT) >= rect.y2)
+            y = rect.y2 - VK_WINDOW_HEIGHT - 2;
+        if (y < rect.y1)
+            y = rect.y1;
+        if ((x + VK_WINDOW_WIDTH) >= rect.x2)
+            x = rect.x2 - VK_WINDOW_WIDTH - 1;
+        if (x < rect.x1)
+            x = rect.x1;
 
 
         XMoveWindow(vkWindow->dpy, vkWindow->window, x, y);
         DisplayVKWindow(vkWindow);
         FcitxUICloseInputWindow(instance);
 
-        FcitxInputContext* ic = FcitxInstanceGetCurrentIC(instance);
-
         if (ic && FcitxInstanceGetCurrentState(instance) == IS_CLOSED)
             FcitxInstanceEnableIM(instance, ic, true);
     } else {
