@@ -33,7 +33,6 @@
 #include "tray.h"
 #include "skin.h"
 #include "classicui.h"
-#include "module/x11/x11stuff.h"
 #include "fcitx-utils/log.h"
 #include "fcitx/frontend.h"
 #include "fcitx/module.h"
@@ -43,7 +42,7 @@
 
 static boolean TrayEventHandler(void *arg, XEvent* event);
 
-void InitTrayWindow(TrayWindow *trayWindow)
+void TrayWindowInit(TrayWindow *trayWindow)
 {
     FcitxClassicUI *classicui = trayWindow->owner;
     Display *dpy = classicui->dpy;
@@ -52,79 +51,83 @@ void InitTrayWindow(TrayWindow *trayWindow)
     if (!classicui->bUseTrayIcon || classicui->isSuspend)
         return;
 
-    InitTray(dpy, trayWindow);
+    if (trayWindow->window == None && trayWindow->dockWindow != None) {
+        XVisualInfo* vi = TrayGetVisual(trayWindow);
+        if (vi && vi->visual) {
+            Window p = DefaultRootWindow(dpy);
+            Colormap colormap = XCreateColormap(dpy, p, vi->visual, AllocNone);
+            XSetWindowAttributes wsa;
+            wsa.background_pixmap = 0;
+            wsa.colormap = colormap;
+            wsa.background_pixel = 0;
+            wsa.border_pixel = 0;
+            trayWindow->window = XCreateWindow(dpy, p, -1, -1, 1, 1,
+                                            0, vi->depth, InputOutput, vi->visual,
+                                            CWBackPixmap | CWBackPixel | CWBorderPixel | CWColormap, &wsa);
+        } else {
+            trayWindow->window = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy),
+                                -1, -1, 1, 1, 0,
+                                BlackPixel(dpy, DefaultScreen(dpy)),
+                                WhitePixel(dpy, DefaultScreen(dpy)));
+            XSetWindowBackgroundPixmap(dpy, trayWindow->window, ParentRelative);
+        }
+        if (trayWindow->window == (Window) NULL)
+            return;
 
-    XVisualInfo* vi = TrayGetVisual(dpy, trayWindow);
-    if (vi && vi->visual) {
-        Window p = DefaultRootWindow(dpy);
-        Colormap colormap = XCreateColormap(dpy, p, vi->visual, AllocNone);
-        XSetWindowAttributes wsa;
-        wsa.background_pixmap = 0;
-        wsa.colormap = colormap;
-        wsa.background_pixel = 0;
-        wsa.border_pixel = 0;
-        trayWindow->window = XCreateWindow(dpy, p, -1, -1, 1, 1,
-                                           0, vi->depth, InputOutput, vi->visual,
-                                           CWBackPixmap | CWBackPixel | CWBorderPixel | CWColormap, &wsa);
-    } else {
-        trayWindow->window = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy),
-                             -1, -1, 1, 1, 0,
-                             BlackPixel(dpy, DefaultScreen(dpy)),
-                             WhitePixel(dpy, DefaultScreen(dpy)));
-        XSetWindowBackgroundPixmap(dpy, trayWindow->window, ParentRelative);
+        XSizeHints size_hints;
+        size_hints.flags = PWinGravity | PBaseSize;
+        size_hints.base_width = trayWindow->size;
+        size_hints.base_height = trayWindow->size;
+        XSetWMNormalHints(dpy, trayWindow->window, &size_hints);
+
+        if (vi && vi->visual)
+            trayWindow->cs_x = cairo_xlib_surface_create(dpy, trayWindow->window, trayWindow->visual.visual, 200, 200);
+        else {
+            Visual *target_visual = DefaultVisual(dpy, iScreen);
+            trayWindow->cs_x = cairo_xlib_surface_create(dpy, trayWindow->window, target_visual, 200, 200);
+        }
+        trayWindow->cs = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 200, 200);
+
+        XSelectInput(dpy, trayWindow->window, ExposureMask | KeyPressMask |
+                    ButtonPressMask | ButtonReleaseMask | StructureNotifyMask
+                    | EnterWindowMask | PointerMotionMask | LeaveWindowMask | VisibilityChangeMask);
+
+        ClassicUISetWindowProperty(classicui, trayWindow->window, FCITX_WINDOW_DOCK, strWindowName);
+
+        TrayFindDock(trayWindow);
     }
-    if (trayWindow->window == (Window) NULL)
-        return;
-
-    XSizeHints size_hints;
-    size_hints.flags = PWinGravity | PBaseSize;
-    size_hints.base_width = trayWindow->size;
-    size_hints.base_height = trayWindow->size;
-    XSetWMNormalHints(dpy, trayWindow->window, &size_hints);
-
-    if (vi && vi->visual)
-        trayWindow->cs_x = cairo_xlib_surface_create(dpy, trayWindow->window, trayWindow->visual.visual, 200, 200);
-    else {
-        Visual *target_visual = DefaultVisual(dpy, iScreen);
-        trayWindow->cs_x = cairo_xlib_surface_create(dpy, trayWindow->window, target_visual, 200, 200);
-    }
-    trayWindow->cs = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 200, 200);
-
-    XSelectInput(dpy, trayWindow->window, ExposureMask | KeyPressMask |
-                 ButtonPressMask | ButtonReleaseMask | StructureNotifyMask
-                 | EnterWindowMask | PointerMotionMask | LeaveWindowMask | VisibilityChangeMask);
-
-    ClassicUISetWindowProperty(classicui, trayWindow->window, FCITX_WINDOW_DOCK, strWindowName);
-
-    TrayFindDock(dpy, trayWindow);
 }
 
-TrayWindow* CreateTrayWindow(FcitxClassicUI *classicui)
+TrayWindow* TrayWindowCreate(FcitxClassicUI *classicui)
 {
     TrayWindow *trayWindow = fcitx_utils_malloc0(sizeof(TrayWindow));
     trayWindow->owner = classicui;
-    InvokeVaArgs(classicui->owner, FCITX_X11, ADDXEVENTHANDLER,
-                 TrayEventHandler, trayWindow);
-    InitTrayWindow(trayWindow);
+    TrayInitAtom(trayWindow);
+    trayWindow->dockWindow = TrayGetDock(trayWindow);
+    FcitxX11AddXEventHandler(classicui->owner, TrayEventHandler, trayWindow);
+    /* We used to init trayWindow here, but now we should delay it */
     return trayWindow;
 }
 
-void ReleaseTrayWindow(TrayWindow *trayWindow)
+void TrayWindowRelease(TrayWindow *trayWindow)
 {
     FcitxClassicUI *classicui = trayWindow->owner;
     Display *dpy = classicui->dpy;
+    trayWindow->bTrayMapped = false;
+    memset(&trayWindow->visual, 0, sizeof(trayWindow->visual));
+
     if (trayWindow->window == None)
         return;
+
     cairo_surface_destroy(trayWindow->cs);
     cairo_surface_destroy(trayWindow->cs_x);
     XDestroyWindow(dpy, trayWindow->window);
     trayWindow->window = None;
     trayWindow->cs = NULL;
     trayWindow->cs_x = NULL;
-    trayWindow->bTrayMapped = false;
 }
 
-void DrawTrayWindow(TrayWindow* trayWindow)
+void TrayWindowDraw(TrayWindow* trayWindow)
 {
     FcitxClassicUI *classicui = trayWindow->owner;
     FcitxSkin *sc = &classicui->skin;
@@ -133,14 +136,15 @@ void DrawTrayWindow(TrayWindow* trayWindow)
     if (!classicui->bUseTrayIcon)
         return;
 
+    if (!trayWindow->bTrayMapped)
+        return;
+
     if (FcitxInstanceGetCurrentState(classicui->owner) == IS_ACTIVE)
         f_state = ACTIVE_ICON;
     else
         f_state = INACTIVE_ICON;
     cairo_t *c;
-    cairo_surface_t *png_surface ;
-    if (!trayWindow->bTrayMapped)
-        return;
+    cairo_surface_t *png_surface;
 
     /* 画png */
     if (f_state) {
@@ -187,7 +191,7 @@ void DrawTrayWindow(TrayWindow* trayWindow)
         }
     } while(0);
 
-    _CAIRO_DESTROY(c);
+    cairo_destroy(c);
 
     XVisualInfo* vi = trayWindow->visual.visual ? &trayWindow->visual : NULL;
     if (!(vi && vi->visual)) {
@@ -205,7 +209,8 @@ void DrawTrayWindow(TrayWindow* trayWindow)
     cairo_rectangle(c, 0, 0, trayWindow->size, trayWindow->size);
     cairo_clip(c);
     cairo_paint(c);
-    _CAIRO_DESTROY(c);
+    cairo_destroy(c);
+    cairo_surface_flush(trayWindow->cs_x);
 }
 
 boolean TrayEventHandler(void *arg, XEvent* event)
@@ -219,17 +224,20 @@ boolean TrayEventHandler(void *arg, XEvent* event)
     switch (event->type) {
     case ClientMessage:
         if (event->xclient.message_type == trayWindow->atoms[ATOM_MANAGER]
-                && event->xclient.data.l[1] == trayWindow->atoms[ATOM_SELECTION]) {
-            if (trayWindow->window == None)
-                InitTrayWindow(trayWindow);
-            TrayFindDock(dpy, trayWindow);
+            && event->xclient.data.l[1] == trayWindow->atoms[ATOM_SELECTION]
+            && trayWindow->dockWindow == None) {
+            if (classicui->notificationItemAvailable)
+                return true;
+            trayWindow->dockWindow = event->xclient.data.l[2];
+            TrayWindowRelease(trayWindow);
+            TrayWindowInit(trayWindow);
             return true;
         }
         break;
 
     case Expose:
         if (event->xexpose.window == trayWindow->window) {
-            DrawTrayWindow(trayWindow);
+            TrayWindowDraw(trayWindow);
         }
         break;
     case ConfigureNotify:
@@ -244,7 +252,7 @@ boolean TrayEventHandler(void *arg, XEvent* event)
                 XSetWMNormalHints(dpy, trayWindow->window, &size_hints);
             }
 
-            DrawTrayWindow(trayWindow);
+            TrayWindowDraw(trayWindow);
             return true;
         }
         break;
@@ -256,11 +264,10 @@ boolean TrayEventHandler(void *arg, XEvent* event)
                 break;
             case Button3: {
                 XlibMenu *mainMenuWindow = classicui->mainMenuWindow;
-                FcitxMenuUpdate(mainMenuWindow->menushell);
-                GetMenuSize(mainMenuWindow);
-                CalMenuWindowPosition(mainMenuWindow, event->xbutton.x_root - event->xbutton.x, event->xbutton.y_root - event->xbutton.y, trayWindow->size);
-                DrawXlibMenu(mainMenuWindow);
-                DisplayXlibMenu(mainMenuWindow);
+                mainMenuWindow->anchor = MA_Tray;
+                mainMenuWindow->trayX = event->xbutton.x_root - event->xbutton.x;
+                mainMenuWindow->trayY = event->xbutton.y_root - event->xbutton.y;
+                XlibMenuShow(mainMenuWindow);
             }
             break;
             }
@@ -270,17 +277,12 @@ boolean TrayEventHandler(void *arg, XEvent* event)
     break;
     case DestroyNotify:
         if (event->xdestroywindow.window == trayWindow->dockWindow) {
-            trayWindow->dockWindow = None;
+            trayWindow->dockWindow = TrayGetDock(trayWindow);
             trayWindow->bTrayMapped = False;
-            ReleaseTrayWindow(trayWindow);
-            return true;
-        }
-        break;
-
-    case ReparentNotify:
-        if (event->xreparent.parent == DefaultRootWindow(dpy) && event->xreparent.window == trayWindow->window) {
-            trayWindow->bTrayMapped = False;
-            ReleaseTrayWindow(trayWindow);
+            TrayWindowRelease(trayWindow);
+            if (trayWindow->dockWindow != None) {
+                TrayWindowInit(trayWindow);
+            }
             return true;
         }
         break;
@@ -288,4 +290,3 @@ boolean TrayEventHandler(void *arg, XEvent* event)
     return false;
 }
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
-
